package jp.sourceforge.talisman.mds.ui.swing;

/*
 * $Id: MdsPane.java 158 2008-07-15 10:45:01Z tama3 $
 */

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JOptionPane;
import javax.swing.JTextPane;
import javax.swing.Scrollable;
import javax.swing.SwingUtilities;

import jp.sourceforge.talisman.i18n.Messages;
import jp.sourceforge.talisman.i18n.ResourceNotFoundException;
import jp.sourceforge.talisman.mds.CalculationFailException;
import jp.sourceforge.talisman.mds.Item;
import jp.sourceforge.talisman.mds.MdsMethod;
import jp.sourceforge.talisman.mds.Table;
import jp.sourceforge.talisman.mds.cluster.Cluster;
import jp.sourceforge.talisman.mds.cluster.ClusteringAlgorithm;
import jp.sourceforge.talisman.mds.cluster.ClusteringMethod;
import jp.sourceforge.talisman.mds.cluster.ClusteringMethodFactory;
import jp.sourceforge.talisman.mds.cluster.ClusteringParameter;
import jp.sourceforge.talisman.mds.cluster.ParameterTypeMismatchException;
import jp.sourceforge.talisman.mds.ui.ColorManager;
import jp.sourceforge.talisman.mds.ui.MdsGraphSetting;
import jp.sourceforge.talisman.mds.ui.mark.DrawerFactory;

/**
 * 
 * @author Haruaki Tamada
 * @version $Revision: 158 $ 
 */
@SuppressWarnings("unchecked")
public class MdsPane extends JTextPane implements Scrollable{
    private static final Dimension MINIMUM_SIZE = new Dimension(100, 100);
    private static final Dimension PREFERRED_SIZE = new Dimension(300, 300);
    private static final long serialVersionUID = -7277586999609831212L;

    private MdsMethod<?> mds;
    private MdsGraphSetting setting;
    private Messages messages;
    private List<ItemsOpenListener> openListeners = new ArrayList<ItemsOpenListener>();
    private List<ItemsSelectionListener> selectionListeners = new ArrayList<ItemsSelectionListener>();

    private ColorManager colorManager = new ColorManager();
    private List<PaintedCluster> clusters = new ArrayList<PaintedCluster>();
    private List<MdsItem> plotItems = new ArrayList<MdsItem>();
    private Rectangle2D drawArea;
    private AffineTransform transform = new AffineTransform();
    private int zoomRatio = 100;
    private Dimension baseSize = PREFERRED_SIZE;

    private Rectangle2D selectedArea = null;
    private boolean selectStart = false;
    private Point2D startPoint = null;
    private PropertyChangeListener propertyChangeListener;

    /**
     * default constructor.
     */
    public MdsPane(MdsMethod<?> mds) throws ResourceNotFoundException{
        this(mds, new MdsGraphSetting());
    }

    public MdsPane(MdsMethod<?> mds, MdsGraphSetting setting) throws ResourceNotFoundException{
        this(mds, setting, new Messages("jp.sourceforge.talisman.mds.ui.MdsViewerPane"));
    }

    public MdsPane(MdsMethod<?> mds, MdsGraphSetting setting, Messages messages) throws ResourceNotFoundException{
        setMessages(messages);
        setMdsMethod(mds);
        setGraphSetting(setting);

        initLayout();
        initListener();
    }

    public void setMessages(Messages messages) throws ResourceNotFoundException{
        if(messages == null){
            throw new ResourceNotFoundException();
        }
        this.messages = messages;
    }

    public Messages getMessages(){
        return messages;
    }

    public void addItemsOpenListener(ItemsOpenListener listener){
        openListeners.add(listener);
    }

    public void addItemsSelectionListener(ItemsSelectionListener listener){
        selectionListeners.add(listener);
    }

    public void clearTransform(){
        selectedArea = null;
        transform.setToIdentity();
        plotItems.clear();
        setSelection(null);
        zoomRatio = 100;
        repaint();
    }

    public Item[] getItems(){
        return mds.getItems();
    }

    public MdsGraphSetting getGraphSetting(){
        return setting;
    }

    /**
     * returns current mds method.  updating returned MdsMethod causes no effects.
     */
    public MdsMethod getMdsMethod(){
        return new MdsMethod(mds);
    }

    public Dimension getPreferredScrollableViewportSize(){
        return getPreferredSize();
    }

    public boolean getScrollableTracksViewportHeight(){
        return false;
    }

    public boolean getScrollableTracksViewportWidth(){
        return false;
    }

    public Item[] getSelectedItems(){
        List<Item> list = new ArrayList<Item>();

        for(MdsItem mdsc: plotItems){
            if(mdsc.isSelected()){
                list.add(mds.getItem(mdsc.getName()));
            }
        }
        return list.toArray(new Item[list.size()]);
    }

    public Table getTable(){
        return mds.getTable();
    }

    public int getZoomRatio(){
        return zoomRatio;
    }

    public void clustering(ClusteringAlgorithm algorithm, ClusteringParameter parameter){
        mds.setLimit(2);
        plotItems.clear();
        clusters.clear();
        try{
            ClusteringMethod method = ClusteringMethodFactory.getInstance().getAlgorithm(algorithm, parameter);
            Cluster[] clusterList = method.clustering(mds.getItems());
            int index = 0;
            for(Cluster c: clusterList){
                clusters.add(new PaintedCluster(c, colorManager.get(index)));
                index++;
            }
            repaint();
        } catch(ParameterTypeMismatchException e){
            JOptionPane.showMessageDialog(this, e.getMessage());
        } catch(CalculationFailException e){
            JOptionPane.showMessageDialog(this, e.getMessage());
        }
    }

    public void mirrorX(){
        selectedArea = null;
        transform.preConcatenate(new AffineTransform(-1f, 0f, 0f, 1f, 0f, 0f));
        plotItems.clear();
        setSelection(null);
        repaint();
    }

    public void mirrorY(){
        selectedArea = null;
        transform.preConcatenate(new AffineTransform(1f, 0f, 0f, -1f, 0f, 0f));
        plotItems.clear();
        setSelection(null);
        repaint();
    }

    public void paintComponent(Graphics graphics){
        super.paintComponent(graphics);
        Graphics2D g = (Graphics2D)graphics;
        MdsGraphSetting mgs = getGraphSetting();

        if(drawArea == null){
            drawArea = calculateDrawArea(mgs);
        }

        Dimension d = getPreferredSize();
        Color initcolor = g.getColor();
        g.setColor(setting.getBackground());
        g.fillRect(0, 0, d.width, d.height);

        if(mgs.isShowGrid())    paintGrid(g, mgs);
        if(mgs.isShowSubGrid()) paintSubGrid(g, mgs);

        if(plotItems.size() == 0){
            Item[] items = mds.getItems();
            double max = calculateMax(items);
            for(int i = 0; i < items.length; i++){
                plotItems.add(calculateItem(items[i], max));
            }
        }
        if(clusters.size() == 0){
            g.setColor(setting.getForeground());
        }
        for(MdsItem c: plotItems){
            if(clusters.size() > 0){
                Item originalItem = c.getOriginal();
                for(PaintedCluster pc: clusters){
                    if(pc.isContain(originalItem)){
                        g.setColor(pc.getColor());
                        break;
                    }
                }
            }
            paintItem(g, mgs, c);
        }

        if(selectedArea != null){
            g.setColor(setting.getSelectedAreaColor());
            g.fill(selectedArea);
        }

        g.setColor(initcolor);
    }

    public void removeItemsOpenListener(ItemsOpenListener listener){
        openListeners.add(listener);
    }

    public void removeItemsSelectionListener(ItemsSelectionListener listener){
        selectionListeners.add(listener);
    }

    public void rotate(double theta){
        transform.preConcatenate(AffineTransform.getRotateInstance(theta));
        plotItems.clear();
        selectedArea = null;
        setSelection(null);
        repaint();
    }

    public void setGraphSetting(MdsGraphSetting setting){
        if(propertyChangeListener == null){
            propertyChangeListener = new PropertyChangeListener(){
                public void propertyChange(PropertyChangeEvent evt){
                    repaint();
                }
            };
        }
        if(setting == null){
            throw new NullPointerException();
        }
        if(this.setting != null){
            this.setting.removePropertyChangeListener(propertyChangeListener);
        }
        this.setting = setting;
        setting.addPropertyChangeListener(propertyChangeListener);
    }

    public void setMdsMethod(MdsMethod<?> mds){
        if(mds == null){
            throw new NullPointerException();
        }
        this.mds = mds;
        transform.setToIdentity();
        plotItems.clear();
    }

    @Override
    public void setPreferredSize(Dimension d){
        this.baseSize = d;
        setZoomRatio(zoomRatio);
    }

    public void setTable(Table table){
        if(table == null){
            throw new NullPointerException();
        }
        if(!table.isValid()){
            throw new IllegalStateException();
        }
        mds.setTable(table);
    }

    public void setZoomRatio(int newZoomRatio){
        if(newZoomRatio < 0){
            throw new IllegalArgumentException("positive value required: " + newZoomRatio);
        }
        zoomRatio = newZoomRatio;

        selectedArea = null;
        drawArea = null;
        plotItems.clear();

        super.setPreferredSize(new Dimension(
            (int)((baseSize.getWidth() / 100d) * zoomRatio),
            (int)((baseSize.getHeight() / 100d) * zoomRatio)
        ));
        if(SwingUtilities.getRootPane(this) != null){
            SwingUtilities.getRootPane(this).repaint();
        }
        updateUI();
    }

    public void transform(AffineTransform tx){
        transform.preConcatenate(tx);
        plotItems.clear();
        selectedArea = null;
        setSelection(null);
        repaint();
    }

    private MdsItem calculateItem(Item c, double max){
        double x = c.get(0).doubleValue();
        double y = c.get(1).doubleValue();

        double w = drawArea.getWidth();
        double h = drawArea.getHeight();

        // x : max = xx : w - 10;
        Point2D point = new Point.Double(x * ((w / 2) - 5) / max, y * ((h / 2) - 5) / max);
        point = transform.transform(point, null);
        MdsItem newItem = new MdsItem(c);
        double ox = drawArea.getX() + drawArea.getWidth() / 2;
        double oy = drawArea.getY() + drawArea.getHeight() / 2;

        newItem.set(0, ox + point.getX());
        newItem.set(1, oy + point.getY());
        newItem.setDrawer(DrawerFactory.getInstance().create(c.getGroupId()));

        return newItem;
    }

    private Rectangle2D calculateDrawArea(MdsGraphSetting mgs){
        Dimension size = getPreferredSize();
        double width = size.getWidth();
        double height = size.getHeight();

        double w = width;
        double h = height;
        if(w < h) h = w;
        else      w = h;
        Rectangle2D drawArea = new Rectangle2D.Double(
            width / 2d - w / 2d, height / 2d - h / 2d, w - 1, h - 1
        );

        return drawArea;
    }

    private double calculateMax(Item[] items){
        double max = 0d;
        for(int i = 0; i < items.length; i++){
            double d1 = Math.abs(items[i].get(0).doubleValue());
            double d2 = Math.abs(items[i].get(1).doubleValue());
            if(max < d1) max = d1;
            if(max < d2) max = d2;
        }
        return max;
    }

    private Rectangle2D calculateSelectedArea(Point2D p1, Point2D p2){
        double x = p1.getX();
        double y = p1.getY();
        if(x > p2.getX()) x = p2.getX();
        if(y > p2.getY()) y = p2.getY();
        return new Rectangle2D.Double(
            x, y, Math.abs(p1.getX() - p2.getX()), Math.abs(p1.getY() - p2.getY())
        );
    }

    private void fireEvent(ItemsOpenEvent e){
        for(ItemsOpenListener listener: openListeners){
            listener.itemOpened(e);
        }
    }

    private void fireEvent(ItemsSelectionEvent e){
        for(ItemsSelectionListener listener: selectionListeners){
            listener.valueChanged(e);
        }
    }

    private void initLayout(){
        setCursor(Cursor.getDefaultCursor());
        setMinimumSize(MINIMUM_SIZE);
        setPreferredSize(PREFERRED_SIZE);
    }

    private void initListener(){
        addComponentListener(new ComponentAdapter(){
            @Override
            public void componentResized(ComponentEvent e){
                selectedArea = null;
                drawArea = null;
                plotItems.clear();
                repaint();
            }
        });

        addKeyListener(new KeyAdapter(){
            @Override
            public void keyReleased(KeyEvent e){
                if(e.getKeyCode() == KeyEvent.VK_ENTER){
                    Item[] c = getSelectedItems();
                    if(selectedArea != null && c.length > 0){
                        fireEvent(new ItemsOpenEvent(MdsPane.this, c));
                    }
                }
            }
        });

        addMouseListener(new MouseAdapter(){
            @Override
            public void mouseClicked(MouseEvent e){
                if(e.getClickCount() == 2){
                    Point2D currentPoint = new Point2D.Double(e.getX(), e.getY());
                    List<MdsItem> items = new ArrayList<MdsItem>();
                    for(MdsItem mdsc: plotItems){
                        Point2D p = new Point2D.Double(mdsc.get(0).doubleValue(), mdsc.get(1).doubleValue());
                        mdsc.setSelected(
                            Math.abs(currentPoint.getX() - p.getX()) < 2d &&
                            Math.abs(currentPoint.getY() - p.getY()) < 2d
                        );
                        if(mdsc.isSelected()){
                            items.add(mdsc);
                        }
                    }
                    fireEvent(new ItemsOpenEvent(MdsPane.this, items.toArray(new MdsItem[items.size()])));
                }
                else{
                    selectedArea = null;
                    setSelection(null);
                    repaint();
                }
            }

            @Override
            public void mousePressed(MouseEvent e){
                if(setting.isSelectable()){
                    selectedArea = null;
                    selectStart = true;
                    startPoint = new Point2D.Double(e.getX(), e.getY());
                }
            }

            @Override
            public void mouseReleased(MouseEvent e){
                selectStart = false;
                setSelection(selectedArea);
                repaint();
            }
        });

        addMouseMotionListener(new MouseMotionAdapter(){
            @Override
            public void mouseDragged(MouseEvent e){
                if(selectStart){
                    Point2D currentPoint = new Point2D.Double(e.getX(), e.getY());
                    selectedArea = calculateSelectedArea(startPoint, currentPoint);
                    setSelection(selectedArea);
                    repaint();
                }
            }
        });
    }

    private void paintItem(Graphics2D g, MdsGraphSetting mgs, MdsItem c){
        double x = c.get(0).doubleValue();
        double y = c.get(1).doubleValue();

        c.paint(g);
        if(mgs.isShowLabels()){
            FontMetrics fm = g.getFontMetrics();
            Rectangle2D r = fm.getStringBounds(c.getName(), g);
            g.drawString(
                c.getName(),
                (float)(x - (r.getWidth() / 2)),
                (float)(y - r.getHeight())
            );
        }
    }

    private void paintGrid(Graphics2D g, MdsGraphSetting mgs){
        g.setColor(setting.getGridColor());
        double x = drawArea.getX();
        double y = drawArea.getY();
        double w = drawArea.getWidth();
        double h = drawArea.getHeight();
        g.draw(drawArea);
        g.draw(new Line2D.Double(x,         y + h / 2, x + w,     y + h / 2));
        g.draw(new Line2D.Double(x + w / 2, y,         x + w / 2, y + h));
    }

    private void paintSubGrid(Graphics2D g, MdsGraphSetting mgs){
        g.setColor(setting.getSubGridColor());

        double x = drawArea.getX();
        double y = drawArea.getY();
        double w = drawArea.getWidth();
        double h = drawArea.getHeight();
        double stepX = w / 8d;
        double stepY = h / 8d;

        for(int i = 1; i < 8; i++){
            if(i != 4){
                g.draw(new Line2D.Double(x,             y + stepY * i, x + w,         y + stepY * i));
                g.draw(new Line2D.Double(x + stepX * i, y,             x + stepX * i, y + h));
            }
        }
    }

    private void setSelection(Rectangle2D rect){
        boolean eventFlag = false;

        List<Item> list = new ArrayList<Item>();
        for(MdsItem mdsc: plotItems){
            boolean flag = mdsc.isSelected();

            Point2D p = new Point2D.Double(mdsc.get(0).doubleValue(), mdsc.get(1).doubleValue());
            mdsc.setSelected(rect != null && rect.contains(p));
            if(mdsc.isSelected()){
                list.add(mds.getItem(mdsc.getName()));
            }
            if(flag != mdsc.isSelected()){
                eventFlag = true;
            }
        }
        // selected item is changed.
        if(eventFlag){
            fireEvent(new ItemsSelectionEvent(this, list.toArray(new Item[list.size()])));
        }
    }
}
